use crate::session::{SessionBackend, SessionClap};

use self::{
    auth_clap::{AuthEngine, ZitadelClap},
    config::ConfigClap,
};

#[derive(clap::Args, Clone, PartialEq, Eq, Debug)]
pub struct AuthClap {
    #[arg(
    env = "AUTH_ENGINE",
    long = "auth-engine",
    requires_ifs = [
        ( "zitadel", "ZitadelClap" )
    ],
    default_value = "noop" )
]
    pub engine: AuthEngine,

    #[arg(
    env = "SESSION_BACKEND",
    long = "session-backend",
    requires_ifs = [
        ( "postgresql", "PostgresqlSessionClap" )
    ],
    default_value = "in-memory" )
]
    pub session_backend: SessionBackend,

    #[clap(flatten)]
    pub zitadel: ZitadelClap,

    #[clap(flatten)]
    pub session: SessionClap,

    #[clap(flatten)]
    pub config: ConfigClap,
}

pub mod auth_clap;
pub mod config;
